﻿/*
  KdbxDowngrade Plugin
  Copyright (C) 2012 Dominik Reichl <dominik.reichl@t-online.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

using KeePass.Forms;
using KeePass.Plugins;
using KeePass.UI;

using KeePassLib;
using KeePassLib.Resources;
using KeePassLib.Serialization;
using KeePassLib.Utility;

namespace KdbxDowngrade
{
	public sealed class KdbxDowngradeExt : Plugin
	{
		private const string ProductName = "KdbxDowngrade";

		private IPluginHost m_host = null;

		public override bool Initialize(IPluginHost host)
		{
			Terminate();

			if(host == null) return false;
			m_host = host;

			m_host.MainWindow.FileSaved += this.OnFileSaved;

			return true;
		}

		public override void Terminate()
		{
			if(m_host == null) return;

			m_host.MainWindow.FileSaved -= this.OnFileSaved;

			m_host = null;
		}

		private void OnFileSaved(object sender, FileSavedEventArgs e)
		{
			try { OnFileSavedPriv(e); }
			catch(Exception ex)
			{
				if(!VistaTaskDialog.ShowMessageBox(ex.Message,
					KdbxDowngradeExt.ProductName, PwDefs.ProductName,
					VtdIcon.Warning, m_host.MainWindow))
					MessageService.ShowWarning(ex);
			}
		}

		private void OnFileSavedPriv(FileSavedEventArgs e)
		{
			if(e == null) { Debug.Assert(false); return; }
			if(e.Database == null) { Debug.Assert(false); return; }

			IOConnectionInfo ioc = e.Database.IOConnectionInfo;
			if(ioc == null) { Debug.Assert(false); return; }

			byte[] pb = IOConnection.ReadFile(ioc);
			if((pb == null) || (pb.Length < 12))
				throw new Exception(ioc.GetDisplayName() +
					MessageService.NewParagraph + KLRes.FileLoadFailed);

			byte[] pbStart = MemUtil.Mid<byte>(pb, 0, 12);
			string strStart = MemUtil.ByteArrayToHexString(pbStart);
			// 2.20 KDBX files start with 03D9A29A67FB4BB501000300;
			// test whether the signature and the major version match
			if(!strStart.StartsWith("03D9A29A67FB4BB5", StrUtil.CaseIgnoreCmp) ||
				!strStart.EndsWith("0300", StrUtil.CaseIgnoreCmp))
				return;

			// Downgrade informational part of the version
			pb[8] = 0;
			pb[9] = 0;

			IOConnectionInfo iocOut = ioc.CloneDeep();
			string strExt = UrlUtil.GetExtension(iocOut.Path);
			string strPath = UrlUtil.StripExtension(iocOut.Path);
			strPath += "_Downgraded";
			if(!string.IsNullOrEmpty(strExt)) strPath += "." + strExt;
			iocOut.Path = strPath;

			Stream s = IOConnection.OpenWrite(iocOut);
			if(s == null)
				throw new Exception(iocOut.GetDisplayName() +
					MessageService.NewParagraph + KLRes.FileSaveFailed);
			s.Write(pb, 0, pb.Length);
			s.Close();
		}
	}
}
